#!/usr/bin/python
# -*- coding: utf-8 -*-
#python 2.7

import sys
import os
import re
import urllib2
import json
import time

username = "admin"
password = ""
domain = "192.168.100.1"

def call_rsp(call_str):
    """return addinfourl"""
    base_url = "http://" + domain 
    passmgr = urllib2.HTTPPasswordMgr()
    passmgr.add_password("hvacrcontrol.com", base_url, username, password)

    auth_handler = urllib2.HTTPDigestAuthHandler(passmgr)
    opener = urllib2.build_opener(auth_handler)
    urllib2.install_opener(opener)

    req = urllib2.Request(base_url + call_str)

    return urllib2.urlopen(req)


def read_cfg_sn():
    """return cfg_sn"""
    rsp = call_rsp("/app/config")
    j = json.loads(rsp.read())

    return j["Result"]["Global"]["cfg_sn"]


def read_summary(cfg_sn, port_id, last_sn):
    """return buf_size, buf_used"""
    rsp = call_rsp("/app/capture_summary?port_id=" + str(port_id) + "&cfg_sn=" + str(cfg_sn))
    j = json.loads(rsp.read())["Result"]

    start = j["start"]
    if last_sn is None:
        """ last_sn is None mean get total bytes used 
        """
        return j["size"], j["used"]
    else:
        """last_sn is not None mean get bytes used after last_sn 
        """
        for hist in j["history"]:
            if hist["head_sn"] == last_sn:
                return j["size"], hist["new_space"]

        """This maybe caused by
        a. invalid last_sn
        b. BACRouter has restarted.
        c. the captured data overflowed buffer, so continous download is unavailable."""
        raise Exception("last sn" + str(last_sn) + " not found")


_filename_pat = re.compile("filename=\"([nir]([0-9]+)_([0-9]+)_([0-9]+).cap)\"")

def read_capture(cfg_sn, port_id, cap_type, last_sn):
    """cap_type: 0=normal format, 1=interval format, 2=raw format
    return data, filename, last_sn
    """
    call_str = "/app/read_capture?port_id=" + str(port_id) + "&cfg_sn=" + str(cfg_sn) + "&cap_type=" + str(cap_type)
    if last_sn is not None:
        call_str = call_str + "&last_sn=" + str(last_sn)

    rsp = call_rsp(call_str)
    data = rsp.read()
    
    match = _filename_pat.search(rsp.info()["content-disposition"])
    if match is None:
        """ something is wrong"""
        return None, None, None

    return data, match.group(1), int(match.group(4))


def main():
    if len(sys.argv) > 1:
        port_id = int(sys.argv[1])
    else:
        port_id = 0


    cfg_sn = read_cfg_sn()
    last_sn = None
    read_time = time.time()

    while 1:
        size, new_space = read_summary(cfg_sn, port_id, last_sn)

        if new_space >= size / 2 or (new_space > 0 and time.time() - read_time > 30*60):
            data, filename, last_sn = read_capture(cfg_sn, port_id, 0, last_sn)
            f = open(filename, "wb")
            f.write(data)
            f.close()
            read_time = time.time()

        time.sleep(10)


if __name__ == '__main__':
    sys.exit(main())

